#include "mymodel.h"

MyModel::MyModel(QObject *parent)
 : QAbstractItemModel( parent )
{
    mRoot = new CItem( "Root", 0 );
    new CItem( "1", mRoot );
    CItem * i2 = new CItem( "2", mRoot );
    new CItem( "2.1", i2 );
    CItem * i22 = new CItem( "2.2", i2 );
    new CItem( "2.2.1", i22 );
    new CItem( "2.2.2", i22 );
    new CItem( "2.2.3", i22 );
    new CItem( "2.3", i2 );
    new CItem( "3", mRoot );
}

QModelIndex MyModel::index(int row, int column, const QModelIndex &parent) const
{
    if( !hasIndex( row, column, parent) ) return QModelIndex();

    CItem * pItem = this->mRoot;
    if( parent.isValid() ) pItem = getItem ( parent );
    CItem * item = pItem->mChildren.at( row );

    return createIndex( row, column, item );
}

QModelIndex MyModel::parent(const QModelIndex &child) const
{
    if( !child.isValid() ) return QModelIndex();
    CItem * item = getItem( child );

    CItem * pItem = item->mParent;
    if( pItem == mRoot ) return QModelIndex();

    CItem * ppItem = pItem->mParent;
    for( std::size_t i = 0; i < ppItem->mChildren.size(); ++i )
        if( ppItem->mChildren.at( i ) == pItem ) return createIndex( i, 0, pItem );

    Q_ASSERT(false);
    return QModelIndex();
}

int MyModel::rowCount(const QModelIndex &parent) const
{
    CItem * item = this->mRoot;
    if( parent.isValid() ) item = getItem( parent );

    return item->mChildren.size();
}

int MyModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 2;
}

QVariant MyModel::data(const QModelIndex &index, int role) const
{
    if( !index.isValid() ) return QVariant();
    CItem * item = getItem( index );

    if( Qt::DisplayRole != role ) return QVariant();

    if( index.column() == 1 )
        return item->mName;
    if( index.column() == 0 )
        return QString("Hey");

    Q_ASSERT(false);
    return QVariant();
}

CItem *MyModel::getItem(const QModelIndex &idx) const
{
    return static_cast<CItem*>( idx.internalPointer() );
}


CItem::CItem(QString name, CItem *parent)
{
    mName = name;
    mParent = parent;
    if( parent ) parent->mChildren.push_back( this );
}
